@@ -16,6 +16,10 @@
 #endif
 #if defined(HAVE_ALLOCA_H)
 #include <alloca.h>
+#else
+#ifdef _AIX
+ #pragma alloca
+#endif
 #endif
 #include <string.h>
 #include <signal.h>
@@ -33,10 +37,6 @@
 #include <netdb.h>
 #endif /* HAVE_GETHOSTBYNAME */
 
-#ifdef SUNOS
-#include <stdlib.h>
-#endif
-
 #include "fetchmail.h"
 #include "tunable.h"
 #include "smtp.h"
@@ -68,6 +68,7 @@ int cmd_daemon; 	/* if --daemon was set */
 char *idfile;		/* UID list file */
 flag versioninfo;	/* emit only version info */
 char *user;		/* the name of the invoking user */
+char *home;
 char *fetchmailhost;	/* the name of the host running fetchmail */
 char *program_name;	/* the name to prefix error messages with */
 
@@ -94,7 +95,6 @@ int main (int argc, char **argv)
 {
     int st, bkgd = FALSE;
     int parsestatus, implicitmode = FALSE;
-    struct passwd *pw;
     struct query *ctl;
     FILE	*lockfp;
     netrc_entry *netrc_list;
@@ -119,7 +119,23 @@ int main (int argc, char **argv)
 
     if (versioninfo)
     {
-	printf("This is fetchmail release %s\n", RELEASE_ID);
+	printf("This is fetchmail release %s", RELEASE_ID);
+#ifdef POP2_ENABLE
+	printf("+POP2");
+#endif /* POP2_ENABLE */
+#ifndef POP3_ENABLE
+	printf("-POP3");
+#endif /* POP3_ENABLE */
+#ifndef IMAP_ENABLE
+	printf("-IMAP");
+#endif /* IMAP_ENABLE */
+#ifdef RPA_ENABLE
+	printf("+RPA");
+#endif /* RPA_ENABLE */
+#ifndef ETRN_ENABLE
+	printf("-ETRN");
+#endif /* ETRN_ENABLE */
+	putchar('\n');
 
 	/* this is an attempt to help remote debugging */
 	system("uname -a");
@@ -131,7 +147,7 @@ int main (int argc, char **argv)
 
     /* set up to do lock protocol */
     if (!getuid())
-	strcpy(tmpbuf, "/var/run/fetchmail.pid");
+	sprintf(tmpbuf, "%s/fetchmail.pid", PID_DIR);
     else {
 	strcpy(tmpbuf, home);
 	strcat(tmpbuf, "/.fetchmail");
@@ -150,6 +166,12 @@ int main (int argc, char **argv)
 	    printf("Lockfile at %s\n", tmpbuf);
 	if (logfile)
 	    printf("Logfile is %s\n", logfile);
+#if defined(HAVE_SYSLOG)
+	if (errors_to_syslog)
+	    printf("Progress messages will be logged via syslog\n");
+#endif
+	if (use_invisible)
+	    printf("Fetchmail will masquerade and will not generate Received\n");
 	for (ctl = querylist; ctl; ctl = ctl->next) {
 	    if (ctl->active && !(implicitmode && ctl->server.skip))
 		dump_params(ctl);
@@ -299,7 +321,7 @@ int main (int argc, char **argv)
      * Maybe time to go to demon mode...
      */
 #if defined(HAVE_SYSLOG)
-    if (use_syslog)
+    if (errors_to_syslog)
     	openlog(program_name, LOG_PID, LOG_MAIL);
 #endif
 
@@ -308,6 +330,14 @@ int main (int argc, char **argv)
 	if (!nodetach)
 	    daemonize(logfile, termhook);
 	error( 0, 0, "starting fetchmail %s daemon ", RELEASE_ID);
+
+	/*
+	 * We'll set up a handler for these when we're sleeping,
+	 * but ignore them otherwise so as not to interrupt a poll.
+	 */
+	signal(SIGUSR1, SIG_IGN);
+	if (poll_interval && !getuid())
+	    signal(SIGHUP, SIG_IGN);
     }
 
     /* beyond here we don't want more than one fetchmail running per user */
@@ -319,20 +349,8 @@ int main (int argc, char **argv)
     signal(SIGPIPE, termhook);
     signal(SIGQUIT, termhook);
 
-    /*
-     * With this simple hack, we make it possible for a foreground 
-     * fetchmail to wake up one in daemon mode.  What we want is the
-     * side effect of interrupting any sleep that may be going on,
-     * forcing fetchmail to re-poll its hosts.
-     */
-    signal(SIGUSR1, donothing);
-
-    /* pacify people who think all system daemons wake up on SIGHUP */
-    if (poll_interval && !getuid())
-	signal(SIGHUP, donothing);
-
     /* here's the exclusion lock */
-    if ( (lockfp = fopen(lockfile,"w")) != NULL ) {
+    if ((lockfp = fopen(lockfile,"w")) != NULL) {
 	fprintf(lockfp,"%d",getpid());
 	if (poll_interval)
 	    fprintf(lockfp," %d", poll_interval);
@@ -351,7 +369,17 @@ int main (int argc, char **argv)
      * reflect the status of that transaction.
      */
     do {
-#ifdef HAVE_RES_SEARCH
+#if defined(HAVE_RES_SEARCH) && defined(USE_TCPIP_FOR_DNS)
+	/*
+	 * This was an efficiency hack that backfired.  The theory
+	 * was that using TCP/IP for DNS queries would get us better
+	 * reliability and shave off some per-UDP-packet costs.
+	 * Unfortunately it interacted badly with diald, which effectively 
+	 * filters out DNS queries over TCP/IP for reasons having to do
+	 * with some obscure kernel problem involving bootstrapping of
+	 * dynamically-addressed links.  I don't understand this mess
+	 * and don't want to, so it's "See ya!" to this hack.
+	 */
 	sethostent(TRUE);	/* use TCP/IP for mailserver queries */
 #endif /* HAVE_RES_SEARCH */
 
@@ -418,11 +446,13 @@ int main (int argc, char **argv)
 
 		querystatus = query_host(ctl);
 
-		if (querystatus == PS_SUCCESS)
+		if (querystatus == PS_SUCCESS) {
 		    successes++;
-
-		if (!check_only)
-		    update_str_lists(ctl);
+#ifdef POP3_ENABLE
+		    if (!check_only)
+		      update_str_lists(ctl);
+#endif  /* POP3_ENABLE */
+		}
 #ifdef	linux
 		if (ctl->server.monitor)
 		    {
@@ -437,7 +467,7 @@ int main (int argc, char **argv)
 	    }
 	}
 
-#ifdef HAVE_RES_SEARCH
+#if defined(HAVE_RES_SEARCH) && defined(USE_TCPIP_FOR_DNS)
 	endhostent();		/* release TCP/IP connection to nameserver */
 #endif /* HAVE_RES_SEARCH */
 
@@ -470,6 +500,17 @@ int main (int argc, char **argv)
 		fprintf(stderr, "fetchmail: sleeping at %s", ctime(&now));
 	    }
 
+	    /*
+	     * With this simple hack, we make it possible for a foreground 
+	     * fetchmail to wake up one in daemon mode.  What we want is the
+	     * side effect of interrupting any sleep that may be going on,
+	     * forcing fetchmail to re-poll its hosts.  The second line is
+	     * for people who think all system daemons wake up on SIGHUP.
+	     */
+	    signal(SIGUSR1, donothing);
+	    if (!getuid())
+		signal(SIGHUP, donothing);
+
 	    /*
 	     * We can't use sleep(3) here because we need an alarm(3)
 	     * equivalent in order to implement server nonresponse timeout.
@@ -498,6 +539,11 @@ int main (int argc, char **argv)
 		}
 	    }
 
+	    /* now lock out interrupts again */
+	    signal(SIGUSR1, SIG_IGN);
+	    if (!getuid())
+		signal(SIGHUP, SIG_IGN);
+
 	    if (outlevel == O_VERBOSE)
 	    {
 		time_t	now;
@@ -521,7 +567,7 @@ static int load_params(int argc, char **argv, int optind)
 {
     int	implicitmode, st;
     struct passwd *pw;
-    struct query def_opts, *ctl, *mp;
+    struct query def_opts, *ctl;
 
     memset(&def_opts, '\0', sizeof(struct query));
     def_opts.smtp_socket = -1;
@@ -529,7 +575,10 @@ static int load_params(int argc, char **argv, int optind)
     def_opts.server.protocol = P_AUTO;
     def_opts.server.timeout = CLIENT_TIMEOUT;
     def_opts.remotename = user;
-    save_str(&def_opts.smtphunt, -1, fetchmailhost);
+    save_str(&def_opts.smtphunt, FALSE, fetchmailhost);
+    save_str(&def_opts.smtphunt, FALSE, "localhost");
+    def_opts.expunge = 1;
+    def_opts.server.envskip = 0;
 
     /* this builds the host list */
     if (prc_parse_file(rcfile, !versioninfo) != 0)
@@ -686,8 +735,10 @@ static int load_params(int argc, char **argv, int optind)
     /* initialize UID handling */
     if (!versioninfo && (st = prc_filecheck(idfile)) != 0)
 	exit(st);
+#ifdef POP3_ENABLE
     else
 	initialize_saved_lists(querylist, idfile);
+#endif /* POP3_ENABLE */
 
     /* if cmd_logfile was explicitly set, use it to override logfile */
     if (cmd_logfile)
@@ -727,8 +778,10 @@ void termhook(int sig)
 	    if (ctl->smtp_socket != -1)
 		SMTP_quit(ctl->smtp_socket);
 
+#ifdef POP3_ENABLE
     if (!check_only)
 	write_saved_lists(querylist, idfile);
+#endif /* POP3_ENABLE */
 
     /* 
      * Craig Metz, the RFC1938 one-time-password guy, points out:
@@ -755,11 +808,18 @@ void termhook(int sig)
 /*
  * Sequence of protocols to try when autoprobing, most capable to least.
  */
+static const int autoprobe[] = 
+{
+#ifdef IMAP_ENABLE
+    P_IMAP,
+#endif /* IMAP_ENABLE */
+#ifdef POP3_ENABLE
+    P_POP3,
+#endif /* POP3_ENABLE */
 #ifdef POP2_ENABLE
-static const int autoprobe[] = {P_IMAP, P_POP3, P_POP2};
-#else
-static const int autoprobe[] = {P_IMAP, P_POP3};
+    P_POP2
 #endif /* POP2_ENABLE */
+};
 
 static int query_host(struct query *ctl)
 /* perform fetch transaction with single host */
@@ -780,7 +840,7 @@ static int query_host(struct query *ctl)
 	for (i = 0; i < sizeof(autoprobe)/sizeof(autoprobe[0]); i++)
 	{
 	    ctl->server.protocol = autoprobe[i];
-	    if ((st = query_host(ctl)) == PS_SUCCESS || st == PS_NOMAIL || st == PS_AUTHFAIL || st == PS_LOCKBUSY)
+	    if ((st = query_host(ctl)) == PS_SUCCESS || st == PS_NOMAIL || st == PS_AUTHFAIL || st == PS_LOCKBUSY || st == PS_SMTP)
 		break;
 	}
 	ctl->server.protocol = P_AUTO;
@@ -797,19 +857,34 @@ static int query_host(struct query *ctl)
     case P_POP3:
     case P_APOP:
     case P_RPOP:
+#ifdef POP3_ENABLE
 	return(doPOP3(ctl));
+#else
+	fprintf(stderr, "POP3 support is not configured.\n");
+	return(PS_PROTOCOL);
+#endif /* POP3_ENABLE */
 	break;
     case P_IMAP:
     case P_IMAP_K4:
+#ifdef IMAP_ENABLE
 	return(doIMAP(ctl));
+#else
+	fprintf(stderr, "IMAP support is not configured.\n");
+	return(PS_PROTOCOL);
+#endif /* IMAP_ENABLE */
 	break;
     case P_ETRN:
+#ifndef ETRN_ENABLE
+	fprintf(stderr, "ETRN support is not configured.\n");
+	return(PS_PROTOCOL);
+#else
 #ifdef HAVE_GETHOSTBYNAME
 	return(doETRN(ctl));
 #else
 	fprintf(stderr, "Cannot support ETRN without gethostbyname(2).\n");
 	return(PS_PROTOCOL);
 #endif /* HAVE_GETHOSTBYNAME */
+#endif /* ETRN_ENABLE */
     default:
 	error(0, 0, "unsupported protocol selected.");
 	return(PS_PROTOCOL);
@@ -830,15 +905,6 @@ void dump_params (struct query *ctl)
 	       ctl->server.interval);
     if (ctl->server.truename)
 	printf("  True name of server is %s.\n", ctl->server.truename);
-    if (ctl->server.akalist)
-    {
-	struct idlist *idp;
-
-	printf("  Predeclared mailserver aliases:");
-	for (idp = ctl->server.akalist; idp; idp = idp->next)
-	    printf(" %s", idp->id);
-	putchar('\n');
-    }
     if (ctl->server.skip || outlevel == O_VERBOSE)
 	printf("  This host will%s be queried when no host is specified.\n",
 	       ctl->server.skip ? " not" : "");
@@ -873,15 +939,6 @@ void dump_params (struct query *ctl)
 	printf(" (default).\n");
     else
 	printf(".\n");
-    if (ctl->server.localdomains)
-    {
-	struct idlist *idp;
-
-	printf("  Local domains:");
-	for (idp = ctl->server.localdomains; idp; idp = idp->next)
-	    printf(" %s", idp->id);
-	putchar('\n');
-    }
 
     if (!ctl->mailboxes->id)
 	printf("  Default mailbox selected.\n");
@@ -931,7 +988,12 @@ void dump_params (struct query *ctl)
     if (ctl->batchlimit > 0)
 	printf("  SMTP message batch limit is %d.\n", ctl->batchlimit);
     else if (outlevel == O_VERBOSE)
-	printf("  No SMTP message batch limit.\n");
+	printf("  No SMTP message batch limit (--batchlimit 0).\n");
+    if (ctl->server.protocol == P_IMAP)
+	if (ctl->expunge > 0)
+	    printf("  Deletion interval between expunges is %d (--expunge %d).\n", ctl->expunge, ctl->expunge);
+	else if (outlevel == O_VERBOSE)
+	    printf("  No expunges (--expunge 0).\n");
     if (ctl->mda)
 	printf("  Messages will be delivered with '%s.'\n", visbuf(ctl->mda));
     else
@@ -940,7 +1002,12 @@ void dump_params (struct query *ctl)
 
 	printf("  Messages will be SMTP-forwarded to:");
 	for (idp = ctl->smtphunt; idp; idp = idp->next)
-	    printf(" %s", idp->id);
+	    if (ctl->server.protocol != P_ETRN || idp->val.num)
+	    {
+		printf(" %s", idp->id);
+		if (!idp->val.num)
+	    	    printf(" (default)");
+	    }
 	printf("\n");
     }
     if (ctl->preconnect)
@@ -963,7 +1030,12 @@ void dump_params (struct query *ctl)
 	for (idp = ctl->localnames; idp; idp = idp->next)
 	    ++count;
 
-	printf("  %d local name(s) recognized.\n", count);
+	if (count > 1 || ctl->wildcard)
+	    printf("  Multi-drop mode: ");
+	else
+	    printf("  Single-drop mode: ");
+
+	printf("%d local name(s) recognized.\n", count);
 	if (outlevel == O_VERBOSE)
 	{
 	    for (idp = ctl->localnames; idp; idp = idp->next)
@@ -975,14 +1047,46 @@ void dump_params (struct query *ctl)
 		fputs("*\n", stdout);
 	}
 
-	printf("  DNS lookup for multidrop addresses is %sabled.\n",
-	       ctl->server.dns ? "en" : "dis");
+	if (count > 1 || ctl->wildcard)
+	{
+	    printf("  DNS lookup for multidrop addresses is %sabled.\n",
+		   ctl->server.dns ? "en" : "dis");
 
-	if (count > 1)
 	    if (ctl->server.envelope == STRING_DISABLED)
 		printf("  Envelope-address routing is disabled\n");
 	    else
-		printf("  Envelope header is assumed to be: %s\n", ctl->server.envelope);
+	    {
+		printf("  Envelope header is assumed to be: %s\n",
+		       ctl->server.envelope ? ctl->server.envelope:"Received");
+		if (ctl->server.envskip > 1 || outlevel >= O_VERBOSE)
+		    printf("  Number of envelope header to be parsed: %d\n",
+			   ctl->server.envskip);
+		if (ctl->server.qvirtual)
+		    printf("  Prefix %s will be removed from user id\n",
+			   ctl->server.qvirtual);
+		else if (outlevel >= O_VERBOSE) 
+		    printf("  No prefix stripping\n");
+	    }
+
+	    if (ctl->server.akalist)
+	    {
+		struct idlist *idp;
+
+		printf("  Predeclared mailserver aliases:");
+		for (idp = ctl->server.akalist; idp; idp = idp->next)
+		    printf(" %s", idp->id);
+		putchar('\n');
+	    }
+	    if (ctl->server.localdomains)
+	    {
+		struct idlist *idp;
+
+		printf("  Local domains:");
+		for (idp = ctl->server.localdomains; idp; idp = idp->next)
+		    printf(" %s", idp->id);
+		putchar('\n');
+	    }
+	}
     }
 #ifdef	linux
     if (ctl->server.interface)
