@@ -18,14 +18,18 @@
 #define		A_PASSWORD	0	/* password or inline authentication */
 #define		A_KERBEROS_V4	1	/* preauthenticate w/ Kerberos V4 */
 
-/* definitions for buffer sizes -- somewhat arbitrary */
-#define		POPBUFSIZE	512	/* per RFC 937 */
+/*
+ * Definitions for buffer sizes.  We get little help on setting maxima
+ * from IMAP RFCs up to 2060, so these are mostly from POP3.
+ */
+#define		HOSTLEN		635	/* max hostname length (RFC1123) */
+#define		POPBUFSIZE	512	/* max length of respone (RFC1939) */
+#define		USERNAMELEN	40	/* max POP3 arg length (RFC1939) */
+#define		IDLEN		128	/* max length of UID (RFC1939) */
+
 #define		MSGBUFSIZE	2048   	/* size of message read buffer */
-#define		HOSTLEN		128	/* max hostname length */
-#define		USERNAMELEN	32	/* max user-name length */
 #define		PASSWORDLEN	64	/* max password length */
 #define		DIGESTLEN	33	/* length of MD5 digest */
-#define		IDLEN		128	/* length of UIDL message ID */
 
 /* exit code values */
 #define		PS_SUCCESS	0	/* successful receipt of messages */
@@ -39,10 +43,11 @@
 #define		PS_EXCLUDE	8	/* client-side exclusion error */
 #define		PS_LOCKBUSY	9	/* server responded lock busy */
 #define		PS_SMTP         10      /* SMTP error */
-#define		PS_UNDEFINED	11	/* something I hadn't thought of */
-#define		PS_TRANSIENT	12	/* transient failure (internal use) */
-#define		PS_REFUSED	13	/* mail refused (internal use) */
-#define		PS_RETAINED	24	/* message retained (internal use) */
+#define		PS_DNS		11	/* fatal DNS error */	
+#define		PS_UNDEFINED	12	/* something I hadn't thought of */
+#define		PS_TRANSIENT	13	/* transient failure (internal use) */
+#define		PS_REFUSED	14	/* mail refused (internal use) */
+#define		PS_RETAINED	15	/* message retained (internal use) */
 
 /* output noise level */
 #define         O_SILENT	0	/* mute, max squelch, etc. */
@@ -89,6 +94,8 @@ struct hostdata		/* shared among all user connections to given server */
     int preauthenticate;		/* preauthentication mode to try */
     int timeout;			/* inactivity timout in seconds */
     char *envelope;			/* envelope address list header */
+    int envskip;			/* skip to numbered envelope header */
+    char *qvirtual;			/* prefix removed from local user id */
     flag skip;				/* suppress poll in implicit mode? */
     flag dns;				/* do DNS lookup on multidrop? */
     flag uidl;				/* use RFC1725 UIDLs? */
@@ -137,8 +144,12 @@ struct query
     int	limit;			/* limit size of retrieved messages */
     int	fetchlimit;		/* max # msgs to get in single poll */
     int	batchlimit;		/* max # msgs to pass in single SMTP session */
+    int	expunge;		/* max # msgs to pass between expunges */
 
     /* unseen, previous state of mailbox (initially from .fetchids) */
+#define UID_KEPT	0	/* this was remembered from a previous run */
+#define UID_DELETED	-1	/* this message has been deleted */
+#define UID_EXPUNGED	-2	/* this message has been expunged */ 
     struct idlist *oldsaved, *newsaved;
 
     /* internal use */
@@ -159,7 +170,6 @@ struct method
     int	port;			/* service port */
     flag tagged;		/* if true, generate & expect command tags */
     flag delimited;		/* if true, accept "." message delimiter */
-    flag force_getsizes;	/* if true, fetch's size return unreliable */
     int (*parse_response)();	/* response_parsing function */
     int (*getauth)();		/* authorization fetcher */
     int (*getrange)();		/* get message range to fetch */
@@ -169,19 +179,26 @@ struct method
     int (*fetch_body)();	/* fetch a given message */
     int (*trail)();		/* eat trailer of a message */
     int (*delete)();		/* delete method */
-    flag (*retain_hdr)();	/* if this returns TRUE, retain message */
-    char *exit_cmd;		/* exit command */
+    int (*logout_cmd)();	/* logout command */
+    flag retry;			/* can getrange poll for new messages? */
 };
 
-#define TAGLEN	6
+/*
+ * Note: tags are generated with an a%04d format from a 1-origin
+ * integer sequence number.  Length 4 permits transaction numbers
+ * up to 9999, so we force rollover with % 10000.  There's no special
+ * reason for this format other than to look like the exmples in the
+ * IMAP RFCs.
+ */
+#define TAGLEN	6		/* 'a' + 4 digits + NUL */
 extern char tag[TAGLEN];
+#define TAGMOD	10000
 
 /* list of hosts assembled from run control file and command line */
 extern struct query cmd_opts, *querylist;
 
 /* what's returned by envquery */
 extern void envquery(int, char **);
-char *user, *home, *fetchmailhost;
 
 /* controls the detail level of status/progress messages written to stderr */
 extern int outlevel;    	/* see the O_.* constants above */
@@ -191,7 +208,8 @@ extern int yydebug;		/* enable parse debugging */
 extern int poll_interval;	/* poll interval in seconds */
 extern flag nodetach;		/* if TRUE, don't detach daemon process */
 extern char *logfile;		/* log file for daemon mode */
-extern flag use_syslog;		/* if --syslog was set */
+extern flag errors_to_syslog;	/* if syslog was set */
+extern flag use_invisible;	/* if invisible was set */
 extern flag quitmode;		/* if --quit was set */
 extern flag check_only;		/* if --check was set */
 extern char *cmd_logfile;	/* if --logfile was set */
@@ -207,37 +225,42 @@ extern char *idfile;		/* path name of UID file */
 extern int linelimit;		/* limit # lines retrieved per site */
 extern flag versioninfo;	/* emit only version info */
 extern char *user;		/* name of invoking user */
+extern char *home;		/* home directory of invoking user */
 extern char *fetchmailhost;	/* the name of the host running fetchmail */
+extern int pass;		/* number of re-polling pass */
 
 /* prototypes for globally callable functions */
+
+/* error.c: Error reporting */
 #if defined(HAVE_STDARG_H)
 void error_init(int foreground);
 void error (int status, int errnum, const char *format, ...);
 void error_build (const char *format, ...);
 void error_complete (int status, int errnum, const char *format, ...);
 void error_at_line (int, int, const char *, unsigned int, const char *, ...);
-void gen_send (int sock, char *, ... );
-int gen_recv(int sock, char *buf, int size);
-int gen_transact (int sock, char *, ... );
 #else
 void error ();
 void error_build ();
 void error_complete ();
 void error_at_line ();
+#endif
+
+/* driver.c: transaction support */
+#if defined(HAVE_STDARG_H)
+void gen_send (int sock, const char *, ... );
+int gen_recv(int sock, char *buf, int size);
+int gen_transact (int sock, char *, ... );
+#else
 void gen_send ();
+int gen_recv();
 int gen_transact ();
 #endif
 
-int do_protocol(struct query *, const struct method *);
-int doPOP2 (struct query *); 
-int doPOP3 (struct query *);
-int doIMAP (struct query *);
-int doETRN (struct query *);
-
-void reply_hack(char *, const char *);
+/* rfc822.c: RFC822 header parsing */
+char *reply_hack(char *, const char *);
 char *nxtaddr(const char *);
 
-/* UID support */
+/* uid.c: UID support */
 void initialize_saved_lists(struct query *, const char *);
 struct idlist *save_str(struct idlist **, int, const char *);
 void free_str_list(struct idlist **);
@@ -246,29 +269,54 @@ void free_str_pair_list(struct idlist **);
 int delete_str(struct idlist **, int);
 int str_in_list(struct idlist **, const char *);
 int str_nr_in_list(struct idlist **, const char *);
+int str_nr_last_in_list(struct idlist **, const char *);
 int count_list( struct idlist **idl );
 char *str_from_nr_list( struct idlist **idl, int number );
 char *str_find(struct idlist **, int);
 char *idpair_find(struct idlist **, const char *);
 void append_str_list(struct idlist **, struct idlist **);
+void expunge_uids(struct query *);
 void update_str_lists(struct query *);
 void write_saved_lists(struct query *, const char *);
 
-struct query *hostalloc(struct query *); 
-int parsecmdline (int, char **, struct query *);
-void optmerge(struct query *, struct query *);
-char *MD5Digest (char *);
-int daemonize(const char *, void (*)(int));
-
-int prc_parse_file(const char *, flag);
+/* rcfile_y.y */
+int prc_parse_file(const char *, const flag);
 int prc_filecheck(const char *);
 
+/* base64.c */
+void to64frombits(unsigned char *, const unsigned char *, int);
+int from64tobits(char *, const char *);
+
+/* interface.c */
 void interface_parse(char *, struct hostdata *);
 void interface_note_activity(struct hostdata *);
 int interface_approve(struct hostdata *);
 
-char *getpassword(char *);
+/* xmalloc.c */
+#if defined(HAVE_VOIDPOINTER)
+#define XMALLOCTYPE void
+#else
+#define XMALLOCTYPE char
+#endif
+XMALLOCTYPE *xmalloc(int);
+XMALLOCTYPE *xrealloc(XMALLOCTYPE *, int);
+char *xstrdup(const char *);
 
+/* protocol driver and methods */
+int do_protocol(struct query *, const struct method *);
+int doPOP2 (struct query *); 
+int doPOP3 (struct query *);
+int doIMAP (struct query *);
+int doETRN (struct query *);
+
+/* miscellanea */
+struct query *hostalloc(struct query *); 
+int parsecmdline (int, char **, struct query *);
+void optmerge(struct query *, struct query *);
+char *MD5Digest (unsigned char *);
+int POP3_auth_rpa(unsigned char *, unsigned char *, int socket);
+int daemonize(const char *, void (*)(int));
+char *getpassword(char *);
 void escapes(const char *, char *);
 char *visbuf(const char *);
 char *showproto(int);
@@ -276,19 +324,6 @@ char *showproto(int);
 void yyerror(const char *);
 int yylex(void);
 
-void to64frombits(unsigned char *, const unsigned char *, int);
-int from64tobits(char *, const char *);
-
-#if defined(HAVE_VOIDPOINTER)
-#define XMALLOCTYPE void
-#else
-#define XMALLOCTYPE char
-#endif
-
-XMALLOCTYPE *xmalloc(int);
-XMALLOCTYPE *xrealloc(XMALLOCTYPE *, int);
-char *xstrdup(const char *);
-
 #define STRING_DISABLED	(char *)-1
 
 /* fetchmail.h ends here */
