@@ -20,19 +20,24 @@
 #include <des.h>
 #define krb_get_err_text(e) (krb_err_txt[e])
 #endif
+#if defined (__FreeBSD__) || defined(__linux__)
+#define krb_get_err_text(e) (krb_err_txt[e])
+#endif
 #include <krb.h>
 #endif /* KERBEROS_V4 */
 
+#ifndef strstr		/* glibc-2.1 declares this as a macro */
 extern char *strstr();	/* needed on sysV68 R3V7.1. */
+#endif /* strstr */
 
 /* imap_version values */
 #define IMAP2		-1	/* IMAP2 or IMAP2BIS, RFC1176 */
 #define IMAP4		0	/* IMAP4 rev 0, RFC1730 */
 #define IMAP4rev1	1	/* IMAP4 rev 1, RFC2060 */
 
-static int count, seen, recent, unseen, deletecount, imap_version;
+static int count, seen, recent, unseen, deletions,expunged, imap_version;
 
-int imap_ok (int sock,  char *argbuf)
+int imap_ok(int sock, char *argbuf)
 /* parse command response */
 {
     char buf [POPBUFSIZE+1];
@@ -60,10 +65,8 @@ int imap_ok (int sock,  char *argbuf)
 	     */
 	    unseen = 0;
 	    for (cp = buf; *cp && !isdigit(*cp); cp++)
-	    {
-		unseen = atoi(cp);
-		break;
-	    }
+		continue;
+	    unseen = atoi(cp);
 	}
 	if (strstr(buf, "FLAGS"))
 	    seen = (strstr(buf, "Seen") != (char *)NULL);
@@ -72,7 +75,8 @@ int imap_ok (int sock,  char *argbuf)
 
     if (tag[0] == '\0')
     {
-	strcpy(argbuf, buf);
+	if (argbuf)
+	    strcpy(argbuf, buf);
 	return(PS_SUCCESS); 
     }
     else
@@ -87,7 +91,8 @@ int imap_ok (int sock,  char *argbuf)
 
 	if (strncmp(cp, "OK", 2) == 0)
 	{
-	    strcpy(argbuf, cp);
+	    if (argbuf)
+		strcpy(argbuf, cp);
 	    return(PS_SUCCESS);
 	}
 	else if (strncmp(cp, "BAD", 2) == 0)
@@ -108,7 +113,7 @@ typedef	long	int32;
 #error Cannot deduce a 32-bit-type
 #endif
 
-static int do_rfc1731(int sock, struct query *ctl, char *buf)
+static int do_rfc1731(int sock, char *truename)
 /* authenticate as per RFC1731 -- note 32-bit integer requirement here */
 {
     int result = 0, len;
@@ -160,7 +165,7 @@ static int do_rfc1731(int sock, struct query *ctl, char *buf)
      * 32-bit number in network byte order.
      */
 
-    strncpy(srvinst, ctl->server.truename, (sizeof srvinst)-1);
+    strncpy(srvinst, truename, (sizeof srvinst)-1);
     srvinst[(sizeof srvinst)-1] = '\0';
     for (p = srvinst; *p; p++) {
       if (isupper(*p)) {
@@ -323,83 +328,77 @@ static int do_rfc1731(int sock, struct query *ctl, char *buf)
 }
 #endif /* KERBEROS_V4 */
 
-int imap_getauth(int sock, struct query *ctl, char *buf)
+int imap_getauth(int sock, struct query *ctl, char *greeting)
 /* apply for connection authorization */
 {
-    char rbuf [POPBUFSIZE+1];
+    char capabilities[POPBUFSIZE+1];
     int ok = 0;
-#ifdef KERBEROS_V4
-    int kerbok = 0;
 
-    if (ctl->server.protocol != P_IMAP_K4) 
-#endif /* KERBEROS_V4 */
-	/* try to get authorized */
-	ok = gen_transact(sock,
-			"LOGIN %s \"%s\"", ctl->remotename, ctl->password);
+    /* probe to see if we're running IMAP4 and can use RFC822.PEEK */
+    gen_send(sock, "CAPABILITY");
+    if ((ok = gen_recv(sock, capabilities, sizeof(capabilities))))
+	return(ok);
+    if (strstr(capabilities, "BAD"))
+    {
+	imap_version = IMAP2;
+	if (outlevel == O_VERBOSE)
+	    error(0, 0, "Protocol identified as IMAP2 or IMAP2BIS");
+    }
+    /* UW-IMAP server 10.173 notifies in all caps */
+    else if (strstr(capabilities, "IMAP4rev1") || strstr(capabilities, "IMAP4REV1"))
+    {
+	imap_version = IMAP4rev1;
+	if (outlevel == O_VERBOSE)
+	    error(0, 0, "Protocol identified as IMAP4 rev 1");
+    }
+    else
+    {
+	imap_version = IMAP4;
+	if (outlevel == O_VERBOSE)
+	    error(0, 0, "Protocol identified as IMAP4 rev 0");
+    }
 
-     if (ok)
-	 return(ok);
+    /* eat the tail of the CAPABILITY response (if any) */
+    if ((peek_capable = (imap_version >= IMAP4)))
+    {
+	char	scratchbuf[POPBUFSIZE];	/* don't clobber capabilities buffer */
 
-     /* probe to see if we're running IMAP4 and can use RFC822.PEEK */
-     gen_send(sock, "CAPABILITY");
-     if ((ok = gen_recv(sock, rbuf, sizeof(rbuf))))
-	 return(ok);
-     if (strstr(rbuf, "BAD"))
-     {
-	 imap_version = IMAP2;
-	 if (outlevel == O_VERBOSE)
-	     error(0, 0, "Protocol identified as IMAP2 or IMAP2BIS");
-     }
-     /* UW-IMAP server 10.173 notifies in all caps */
-     else if (strstr(rbuf, "IMAP4rev1") || strstr(rbuf, "IMAP4REV1"))
-     {
-	 imap_version = IMAP4rev1;
-	 if (outlevel == O_VERBOSE)
-	     error(0, 0, "Protocol identified as IMAP4 rev 1");
-     }
-     else
-     {
-	 imap_version = IMAP4;
-	 if (outlevel == O_VERBOSE)
-	     error(0, 0, "Protocol identified as IMAP4 rev 0");
-     }
-
-     peek_capable = (imap_version >= IMAP4);
+	if ((ok = gen_recv(sock, scratchbuf, sizeof(scratchbuf))))
+	    return(ok);
+    }
 
 #ifdef KERBEROS_V4
-     if (strstr(rbuf, "AUTH=KERBEROS_V4"))
-     {
-	 kerbok++;
-	 if (outlevel == O_VERBOSE)
-		error(0, 0, "KERBEROS_V4 authentication is supported");
-     }
+    if (strstr(capabilities, "AUTH=KERBEROS_V4"))
+    {
+	if (outlevel == O_VERBOSE)
+	    error(0, 0, "KERBEROS_V4 authentication is supported");
 
-     /* eat OK response */
-     if ((ok = gen_recv(sock, rbuf, sizeof(rbuf))))
-	 return(ok);
-
-     if (!strstr(rbuf, "OK"))
- 	 return(PS_AUTHFAIL);
- 
-     if ((imap_version >= IMAP4) && (ctl->server.protocol == P_IMAP_K4))
-     {
-	 if (!kerbok)
-	 {
-	     error(0, -1, "Required KERBEROS_V4 capability not supported by server");
-	     return(PS_AUTHFAIL);
-	 }
-
-	 if ((ok = do_rfc1731(sock, ctl, buf)))
-	 {
-	     if (outlevel == O_VERBOSE)
-		 error(0, 0, "IMAP> *");
-	     SockWrite(sock, "*\r\n", 3);
-	     return(ok);
-	 }
-     }
+	if (ctl->server.protocol == P_IMAP_K4)
+	{
+	    if ((ok = do_rfc1731(sock, ctl->server.truename)))
+	    {
+		if (outlevel == O_VERBOSE)
+		    error(0, 0, "IMAP> *");
+		SockWrite(sock, "*\r\n", 3);
+	    }
+	    
+	    return(ok);
+	}
+	/* else fall through to ourdinary AUTH=LOGIN case */
+    }
+    else if (ctl->server.protocol == P_IMAP_K4)
+    {
+	error(0,-1, "Required KERBEROS_V4 capability not supported by server");
+	return(PS_AUTHFAIL);
+    }
 #endif /* KERBEROS_V4 */
 
-     return(PS_SUCCESS);
+    /* try to get authorized in the ordinary (AUTH=LOGIN) way */
+    ok = gen_transact(sock, "LOGIN %s \"%s\"", ctl->remotename, ctl->password);
+    if (ok)
+	return(ok);
+
+    return(PS_SUCCESS);
 }
 
 static int imap_getrange(int sock, 
@@ -412,11 +411,34 @@ static int imap_getrange(int sock,
 
     /* find out how many messages are waiting */
     recent = unseen = -1;
-    ok = gen_transact(sock, "SELECT %s", folder ? folder : "INBOX");
-    if (ok != 0)
+
+    if (pass > 1)
     {
-	error(0, 0, "mailbox selection failed");
-	return(ok);
+	/* 
+	 * We have to have an expunge here, otherwise the re-poll will
+	 * infinite-loop picking up un-expunged message.
+	 */
+	ok = 0;
+	if (deletions && ctl->expunge > 1)
+	    ok = gen_transact(sock, "EXPUNGE");
+#ifdef IMAP_UID	/* not used */
+	if (!ok)
+	    expunge_uids(ctl);
+#endif /* IMAP_UID */
+	if (ok || gen_transact(sock, "NOOP"))
+	{
+	    error(0, 0, "re-poll failed");
+	    return(ok);
+	}
+    }
+    else
+    {
+	ok = gen_transact(sock, "SELECT %s", folder ? folder : "INBOX");
+	if (ok != 0)
+	{
+	    error(0, 0, "mailbox selection failed");
+	    return(ok);
+	}
     }
 
     *countp = count;
@@ -435,7 +457,7 @@ static int imap_getrange(int sock,
     else
 	*newp = -1;		/* should never happen, RECENT is mandatory */ 
 
-    deletecount = 0;
+    expunged = deletions = 0;
 
     return(PS_SUCCESS);
 }
@@ -446,7 +468,7 @@ static int imap_getsizes(int sock, int count, int *sizes)
     char buf [POPBUFSIZE+1];
 
     gen_send(sock, "FETCH 1:%d RFC822.SIZE", count);
-    while (SockRead(sock, buf, sizeof(buf)))
+    for (;;)
     {
 	int num, size, ok;
 
@@ -456,8 +478,6 @@ static int imap_getsizes(int sock, int count, int *sizes)
 	    break;
 	else if (sscanf(buf, "* %d FETCH (RFC822.SIZE %d)", &num, &size) == 2)
 	    sizes[num - 1] = size;
-	else
-	    sizes[num - 1] = -1;
     }
 
     return(PS_SUCCESS);
@@ -469,7 +489,7 @@ static int imap_is_old(int sock, struct query *ctl, int number)
     int ok;
 
     /* expunges change the fetch numbers */
-    number -= deletecount;
+    number -= expunged;
 
     if ((ok = gen_transact(sock, "FETCH %d FLAGS", number)) != 0)
 	return(PS_ERROR);
@@ -484,7 +504,7 @@ static int imap_fetch_headers(int sock, struct query *ctl,int number,int *lenp)
     int	num;
 
     /* expunges change the fetch numbers */
-    number -= deletecount;
+    number -= expunged;
 
     /*
      * This is blessed by RFC 1176, RFC1730, RFC2060.
@@ -514,7 +534,7 @@ static int imap_fetch_body(int sock, struct query *ctl, int number, int *lenp)
     int	num;
 
     /* expunges change the fetch numbers */
-    number -= deletecount;
+    number -= expunged;
 
     /*
      * If we're using IMAP4, we can fetch the message without setting its
@@ -572,7 +592,7 @@ static int imap_trail(int sock, struct query *ctl, int number)
 /* discard tail of FETCH response after reading message text */
 {
     /* expunges change the fetch numbers */
-    /* number -= deletecount; */
+    /* number -= expunged; */
 
     for (;;)
     {
@@ -581,7 +601,9 @@ static int imap_trail(int sock, struct query *ctl, int number)
 
 	if ((ok = gen_recv(sock, buf, sizeof(buf))))
 	    return(ok);
-	if (strstr(buf, "OK FETCH"))
+
+	/* UW IMAP returns "OK FETCH", Cyrus returns "OK Completed" */
+	if (strstr(buf, "OK"))
 	    break;
     }
 
@@ -594,7 +616,7 @@ static int imap_delete(int sock, struct query *ctl, int number)
     int	ok;
 
     /* expunges change the fetch numbers */
-    number -= deletecount;
+    number -= expunged;
 
     /*
      * Use SILENT if possible as a minor throughput optimization.
@@ -608,29 +630,43 @@ static int imap_delete(int sock, struct query *ctl, int number)
 	return(ok);
 
     /*
-     * We do an expunge after each message, rather than just before quit,
-     * so that a line hit during a long session won't result in lots of
-     * messages being fetched again during the next session.
+     * We do an expunge after ctl->expunge messages, rather than
+     * just before quit, so that a line hit during a long session
+     * won't result in lots of messages being fetched again during
+     * the next session.
      */
-    if ((ok = gen_transact(sock, "EXPUNGE")))
-	return(ok);
+    if (ctl->expunge > 0 && (++deletions % ctl->expunge) == 0)
+    {
+	if ((ok = gen_transact(sock, "EXPUNGE")))
+	    return(ok);
 
-    deletecount++;
+#ifdef IMAP_UID	/* not used */
+	expunge_uids(ctl);
+#endif /* IMAP_UID */
+
+	expunged = deletions;
+    }
 
     return(PS_SUCCESS);
 }
 
-static flag imap_retain_check(int num, char *buf)
-/* is this a special message that should be retained? */
+static int imap_logout(int sock, struct query *ctl)
+/* send logout command */
 {
-    /*
-     * The University of Washington IMAP server (the reference
-     * implementation of IMAP4 written by Mark Crispin) relies
-     * on being able to keep base-UID information in a special
-     * message at the head of the mailbox.  This message should
-     * neither be deleted nor forwarded.
-     */
-    return (num == 1 && !strncasecmp(buf, "X-IMAP:", 7));
+    /* if expunges after deletion have been suppressed, ship one now */
+    if (ctl->expunge == 0 && deletions)
+    {
+	int	ok;
+
+	if ((ok = gen_transact(sock, "EXPUNGE")))
+	    return(ok);
+
+#ifdef IMAP_UID	/* not used */
+	expunge_uids(ctl);
+#endif /* IMAP_UID */
+    }
+
+    return(gen_transact(sock, "LOGOUT"));
 }
 
 const static struct method imap =
@@ -639,18 +675,17 @@ const static struct method imap =
     143,		/* standard IMAP2bis/IMAP4 port */
     TRUE,		/* this is a tagged protocol */
     FALSE,		/* no message delimiter */
-    FALSE,		/* fetch response size is reliable */ 
     imap_ok,		/* parse command response */
     imap_getauth,	/* get authorization */
     imap_getrange,	/* query range of messages */
-    imap_getsizes,	/* grab message sizes */
+    imap_getsizes,	/* get sizes of messages (used for --limit option */
     imap_is_old,	/* no UID check */
     imap_fetch_headers,	/* request given message headers */
     imap_fetch_body,	/* request given message body */
     imap_trail,		/* eat message trailer */
     imap_delete,	/* delete the message */
-    imap_retain_check,	/* leave this message alone? */
-    "LOGOUT",		/* the IMAP exit command */
+    imap_logout,	/* expunge and exit */
+    TRUE,		/* yes, we can re-poll */
 };
 
 int doIMAP(struct query *ctl)
