@@ -23,7 +23,9 @@
 #include <unistd.h>
 #endif
 #include <sys/ioctl.h>
+#include <sys/socket.h>
 #include <netinet/in.h>
+#include <arpa/inet.h>
 #include <net/if.h>
 #include "config.h"
 #include "fetchmail.h"
@@ -46,6 +48,7 @@ static int _get_ifinfo_(int socket_fd, FILE *stats_file, const char *ifname,
 	struct ifreq request;
 	char *cp, buffer[256];
 	int found = 0;
+	int counts[4];
 
 	/* initialize result */
 	memset((char *) ifinfo, 0, sizeof(ifinfo_t));
@@ -56,8 +59,17 @@ static int _get_ifinfo_(int socket_fd, FILE *stats_file, const char *ifname,
 		if (!strncmp(cp, ifname, namelen) &&
 				cp[namelen] == ':') {
 			cp += namelen + 1;
-			sscanf(cp, "%d %*d %*d %*d %*d %d %*d %*d %*d %*d %*d",
-				&ifinfo->rx_packets, &ifinfo->tx_packets);
+			if (sscanf(cp, "%d %d %*d %*d %*d %d %*d %d %*d %*d"
+			       " %*d %*d %d",counts, counts+1, counts+2, 
+				   counts+3,&found)>4) { /* found = dummy */
+			        /* newer kernel with byte counts */
+			        ifinfo->rx_packets=counts[1];
+			        ifinfo->tx_packets=counts[3];
+			} else {
+			        /* older kernel, no byte counts */
+			        ifinfo->rx_packets=counts[0];
+			        ifinfo->tx_packets=counts[2];
+			}
                         found = 1;
 		}
 	}
@@ -110,6 +122,34 @@ static int get_ifinfo(const char *ifname, ifinfo_t *ifinfo)
 	return(result);
 }
 
+#ifndef HAVE_INET_ATON
+/*
+ * Note: This is not a true replacement for inet_aton(), as it won't
+ * do the right thing on "255.255.255.255" (which translates to -1 on
+ * most machines).  Fortunately this code will be used only if you're
+ * on an older Linux that lacks a real implementation.
+ */
+#ifdef HAVE_NETINET_IN_SYSTM_H
+# include <sys/types.h>
+# include <netinet/in_systm.h>
+#endif
+
+#include <netinet/in.h>
+#include <netinet/ip.h>
+#include <arpa/inet.h>
+#include <string.h>
+
+static int inet_aton(const char *cp, struct in_addr *inp) {
+    long addr;
+
+    addr = inet_addr(cp);
+    if (addr == ((long) -1)) return 0;
+
+    memcpy(inp, &addr, sizeof(addr));
+    return 1;
+}
+#endif HAVE_INET_ATON
+
 void interface_parse(char *buf, struct hostdata *hp)
 /* parse 'interface' specification */
 {
