@@ -25,34 +25,37 @@
 #define LA_NODETACH	7
 #define LA_QUIT		8
 #define LA_LOGFILE	9
-#define LA_SYSLOG	10
-#define LA_RCFILE	11
-#define LA_IDFILE	12
-#define LA_PROTOCOL	13
-#define LA_UIDL		14
-#define LA_PORT		15
-#define LA_AUTHENTICATE	16
-#define LA_TIMEOUT	17
-#define LA_ENVELOPE	18
-#define LA_USERNAME	19
-#define LA_ALL          20
-#define LA_NOKEEP	21
-#define	LA_KEEP		22
-#define LA_FLUSH        23
-#define LA_NOREWRITE	24
-#define LA_LIMIT	25
-#define LA_FOLDER	26
-#define LA_SMTPHOST	27
-#define LA_BATCHLIMIT	28
-#define LA_FETCHLIMIT	29
-#define LA_MDA		30
-#define LA_INTERFACE    31
-#define LA_MONITOR      32
-#define LA_YYDEBUG	33
+#define LA_INVISIBLE	10
+#define LA_SYSLOG	11
+#define LA_RCFILE	12
+#define LA_IDFILE	13
+#define LA_PROTOCOL	14
+#define LA_UIDL		15
+#define LA_PORT		16
+#define LA_AUTHENTICATE	17
+#define LA_TIMEOUT	18
+#define LA_ENVELOPE	19
+#define LA_USERNAME	20
+#define LA_ALL          21
+#define LA_NOKEEP	22
+#define	LA_KEEP		23
+#define LA_FLUSH        24
+#define LA_NOREWRITE	25
+#define LA_LIMIT	26
+#define LA_FOLDER	27
+#define LA_SMTPHOST	28
+#define LA_BATCHLIMIT	29
+#define LA_FETCHLIMIT	30
+#define LA_EXPUNGE	31
+#define LA_MDA		32
+#define LA_INTERFACE    33
+#define LA_MONITOR      34
+#define LA_YYDEBUG	35
+#define LA_QVIRTUAL     36
 
-/* options still left: CDegGhHjJoOqQRTUwWxXYzZ */
+/* options still left: CDgGhHjJoORTUwWxXYzZ */
 static const char *shortoptions = 
-	"?Vcsvd:NqL:f:i:p:UP:A:t:E:u:akKFnl:r:S:b:B:m:I:M:y";
+	"?Vcsvd:NqL:f:i:p:UP:A:t:E:Q:u:akKFnl:r:S:b:B:e:m:I:M:y";
 
 static const struct option longoptions[] = {
 /* this can be const because all flag fields are 0 and will never get set */
@@ -65,6 +68,7 @@ static const struct option longoptions[] = {
   {"nodetach",	no_argument,	   (int *) 0, LA_NODETACH    },
   {"quit",	no_argument,	   (int *) 0, LA_QUIT        },
   {"logfile",	required_argument, (int *) 0, LA_LOGFILE     },
+  {"invisible",	no_argument,	   (int *) 0, LA_INVISIBLE   },
   {"syslog",	no_argument,	   (int *) 0, LA_SYSLOG      },
   {"fetchmailrc",required_argument,(int *) 0, LA_RCFILE      },
   {"idfile",	required_argument, (int *) 0, LA_IDFILE      },
@@ -76,6 +80,7 @@ static const struct option longoptions[] = {
   {"auth",	required_argument, (int *) 0, LA_AUTHENTICATE},
   {"timeout",	required_argument, (int *) 0, LA_TIMEOUT     },
   {"envelope",	required_argument, (int *) 0, LA_ENVELOPE    },
+  {"qvirtual",	required_argument, (int *) 0, LA_QVIRTUAL    },
 
   {"user",	required_argument, (int *) 0, LA_USERNAME    },
   {"username",  required_argument, (int *) 0, LA_USERNAME    },
@@ -91,6 +96,7 @@ static const struct option longoptions[] = {
   {"smtphost",	required_argument, (int *) 0, LA_SMTPHOST    },
   {"batchlimit",required_argument, (int *) 0, LA_BATCHLIMIT  },
   {"fetchlimit",required_argument, (int *) 0, LA_FETCHLIMIT  },
+  {"expunge",   required_argument, (int *) 0, LA_EXPUNGE     },
   {"mda",	required_argument, (int *) 0, LA_MDA         },
 
 #ifdef	linux
@@ -105,8 +111,8 @@ static const struct option longoptions[] = {
 
 int parsecmdline (argc, argv, ctl)
 /* parse and validate the command line options */
-int argc;			/* argument count */
-char **argv;			/* argument strings */
+int argc;		/* argument count */
+char **argv;		/* argument strings */
 struct query *ctl;	/* option record to be initialized */
 {
     /*
@@ -165,6 +171,9 @@ struct query *ctl;	/* option record to be initialized */
 	case LA_LOGFILE:
 	    cmd_logfile = optarg;
 	    break;
+	case LA_INVISIBLE:
+	    use_invisible = TRUE;
+	    break;
 	case 'f':
 	case LA_RCFILE:
 	    rcfile = (char *) xmalloc(strlen(optarg)+1);
@@ -236,6 +245,10 @@ struct query *ctl;	/* option record to be initialized */
 	case LA_ENVELOPE:
 	    ctl->server.envelope = xstrdup(optarg);
 	    break;
+	case 'Q':    
+	case LA_QVIRTUAL:
+	    ctl->server.qvirtual = xstrdup(optarg);
+	    break;
 
 	case 'u':
 	case LA_USERNAME:
@@ -272,7 +285,7 @@ struct query *ctl;	/* option record to be initialized */
 	    strcpy(buf, optarg);
 	    cp = strtok(buf, ",");
 	    do {
-		save_str(&ctl->mailboxes, -1, optarg);
+		save_str(&ctl->mailboxes, -1, cp);
 	    } while
 		((cp = strtok((char *)NULL, ",")));
 	    break;
@@ -281,7 +294,7 @@ struct query *ctl;	/* option record to be initialized */
 	    strcpy(buf, optarg);
 	    cp = strtok(buf, ",");
 	    do {
-		save_str(&ctl->smtphunt, -1, optarg);
+		save_str(&ctl->smtphunt, TRUE, cp);
 	    } while
 		((cp = strtok((char *)NULL, ",")));
 	    ocount++;
@@ -298,6 +311,12 @@ struct query *ctl;	/* option record to be initialized */
 	    if (ctl->fetchlimit == 0)
 		ctl->fetchlimit = -1;
 	    break;
+	case 'e':
+	case LA_EXPUNGE:
+	    ctl->expunge = atoi(optarg);
+	    if (ctl->expunge == 0)
+		ctl->expunge = -1;
+	    break;
 	case 'm':
 	case LA_MDA:
 	    ctl->mda = xstrdup(optarg);
@@ -321,7 +340,7 @@ struct query *ctl;	/* option record to be initialized */
 	    break;
 
 	case LA_SYSLOG:
-	    use_syslog = TRUE;
+	    errors_to_syslog = TRUE;
 	    break;
 
 	case '?':
@@ -346,6 +365,7 @@ struct query *ctl;	/* option record to be initialized */
 	fputs("  -q, --quit        kill daemon process\n", stderr);
 	fputs("  -L, --logfile     specify logfile name\n", stderr);
 	fputs("      --syslog      use syslog(3) for most messages when running as a daemon\n", stderr);
+	fputs("      --invisible   suppress Received line & enable host spoofing\n", stderr);
 	fputs("  -f, --fetchmailrc specify alternate run control file\n", stderr);
 	fputs("  -i, --idfile      specify alternate UIDs file\n", stderr);
 #ifdef	linux
@@ -363,6 +383,7 @@ struct query *ctl;	/* option record to be initialized */
 	fputs("  -A, --auth        authentication type (password or kerberos)\n",stderr);
 	fputs("  -t, --timeout     server nonresponse timeout\n",stderr);
 	fputs("  -E, --envelope    envelope address header\n",stderr);
+	fputs("  -Q, --qvirtual    prefix to remove from local user id\n",stderr);
 
 	fputs("  -u, --username    specify users's login on server\n", stderr);
 	fputs("  -a, --all         retrieve old and new messages\n", stderr);
@@ -375,6 +396,7 @@ struct query *ctl;	/* option record to be initialized */
 	fputs("  -S, --smtphost    set SMTP forwarding host\n", stderr);
 	fputs("  -b, --batchlimit  set batch limit for SMTP connections\n", stderr);
 	fputs("  -B, --fetchlimit  set fetch limit for server connections\n", stderr);
+	fputs("  -e, --expunge     set max deletions between expunges\n", stderr);
 	fputs("  -r, --folder      specify remote folder name\n", stderr);
 	return(-1);
     }
