@@ -114,13 +114,18 @@ static int pop2_trail(int sock, struct query *ctl, int number)
     return(gen_transact(sock, ctl->keep ? "ACKS" : "ACKD"));
 }
 
+static int pop2_logout(int sock, struct query *ctl)
+/* send logout command */
+{
+    return(gen_transact(sock, "QUIT"));
+}
+
 const static struct method pop2 =
 {
     "POP2",				/* Post Office Protocol v2 */
     109,				/* standard POP2 port */
     FALSE,				/* this is not a tagged protocol */
     FALSE,				/* does not use message delimiter */
-    FALSE,				/* no getsizes method */
     pop2_ok,				/* parse command response */
     pop2_getauth,			/* get authorization */
     pop2_getrange,			/* query range of messages */
@@ -130,8 +135,8 @@ const static struct method pop2 =
     NULL,				/* no way to fetch body alone */
     pop2_trail,				/* eat message trailer */
     NULL,				/* no POP2 delete method */
-    NULL,				/* no retained-header check */
-    "QUIT",				/* the POP2 exit command */
+    pop2_logout,			/* log out, we're done */
+    FALSE,				/* no, we can't re-poll */
 };
 
 int doPOP2 (struct query *ctl)
