@@ -5,7 +5,7 @@
  */
 
 #include  "config.h"
-
+#ifdef POP3_ENABLE
 #include  <stdio.h>
 #include  <string.h>
 #include  <ctype.h>
@@ -23,11 +23,9 @@
 #include  <opie.h>
 #endif /* HAVE_LIBOPIE */
 
-#define PROTOCOL_ERROR	{error(0, 0, "protocol error"); return(PS_ERROR);}
-
-#define LOCKBUSY_ERROR	{error(0, 0, "lock busy!  Is another session active?"); return(PS_LOCKBUSY);}
-
+#ifndef strstr		/* glibc-2.1 declares this as a macro */
 extern char *strstr();	/* needed on sysV68 R3V7.1. */
+#endif /* strstr */
 
 static int last;
 
@@ -52,7 +50,9 @@ int pop3_ok (int sock, char *argbuf)
 
 	while (isalpha(*bufp))
 	    bufp++;
-	*(bufp++) = '\0';
+
+	if (*bufp)
+	  *(bufp++) = '\0';
 
 	if (strcmp(buf,"+OK") == 0)
 	{
@@ -76,7 +76,9 @@ int pop3_ok (int sock, char *argbuf)
 	    if (strstr(bufp,"lock")||strstr(bufp,"Lock")||strstr(bufp,"LOCK"))
 		ok = PS_LOCKBUSY;
 	    else
-		ok = PS_ERROR;
+		ok = PS_AUTHFAIL;
+	    if (*bufp)
+	      error(0,0,bufp);
 	}
 	else
 	    ok = PS_PROTOCOL;
@@ -92,16 +94,78 @@ int pop3_getauth(int sock, struct query *ctl, char *greeting)
 /* apply for connection authorization */
 {
     int ok;
+    char *start,*end;
+    char *msg;
 #if HAVE_LIBOPIE
     char *challenge;
 #endif /* HAVE_LIBOPIE */
 
-    /* build MD5 digest from greeting timestamp + password */
-    if (ctl->server.protocol == P_APOP) 
-    {
-	char *start,*end;
-	char *msg;
+    switch (ctl->server.protocol) {
+    case P_POP3:
+	 ok = gen_transact(sock, "USER %s", ctl->remotename);
 
+	 if (ok != 0)
+#ifdef RPA_ENABLE
+            /* if CompuServe rejected our userid, try RPA */
+            if (strstr(greeting, "csi.com"))
+            {
+                /* AUTH command should return a list of available mechanisms */
+                if (gen_transact(sock, "AUTH") == 0)
+                {
+                    char buffer[10];
+                    flag has_rpa = FALSE;
+                    flag authenticated = FALSE;
+
+                    while ((ok = gen_recv(sock, buffer, sizeof(buffer))) == 0)
+                    {
+
+                        if (buffer[0] == '.')
+                            break;
+                        if (strncasecmp(buffer, "rpa", 3) == 0)
+                            has_rpa = TRUE;
+                    }
+                    if(has_rpa) {
+                        if (POP3_auth_rpa(ctl->remotename,
+                                          ctl->password, sock) == PS_SUCCESS)
+                        {
+                            authenticated = TRUE;
+                            break;
+                        }
+                    }
+                    if (authenticated) {
+                        /* we could return(PS_SUCCESS) here */
+                        ok = PS_SUCCESS;
+                        break;
+                    }
+                }
+            }
+	    else	/* not a CompuServe host, fail in the ordinary way */
+#endif /* RPA_ENABLE */
+		break;
+
+#if defined(HAVE_LIBOPIE) && defined(OPIE_ENABLE)
+	/* see RFC1938: A One-Time Password System */
+	if (challenge = strstr(lastok, "otp-"))
+	{
+	    char response[OPIE_RESPONSE_MAX+1];
+
+	    if (opiegenerator(challenge, ctl->password, response))
+	    {
+		ok = PS_ERROR;
+		break;
+	    }
+
+	    ok = gen_transact(sock, "PASS %s", response);
+	    break;
+	}
+#endif /* defined(HAVE_LIBOPIE) && defined(OPIE_ENABLE) */
+
+	/* ordinary validation, no one-time password or RPA */ 
+	ok = gen_transact(sock, "PASS %s", ctl->password);
+	break;
+
+    case P_APOP:
+	/* build MD5 digest from greeting timestamp + password */
 	/* find start of timestamp */
 	for (start = greeting;  *start != 0 && *start != '<';  start++)
 	    continue;
@@ -127,76 +191,41 @@ int pop3_getauth(int sock, struct query *ctl, char *greeting)
 
 	strcpy(ctl->digest, MD5Digest(msg));
 	free(msg);
-    }
 
-    switch (ctl->server.protocol) {
-    case P_POP3:
-	if ((gen_transact(sock, "USER %s", ctl->remotename)) != 0)
-	    PROTOCOL_ERROR
-
-#if defined(HAVE_LIBOPIE) && defined(OPIE_ENABLE)
-	/* see RFC1938: A One-Time Password System */
-	if (challenge = strstr(lastok, "otp-"))
-	{
-	    char response[OPIE_RESPONSE_MAX+1];
-
-	    /*
-	     * Special case in case we're running Craig Metz's
-	     * OPIE daemon.  Code in opiegenerator() will detect this.
-	     */
-	    if (ctl->password && !strcmp(ctl->password, "opie"))
-	    {
-		if (ok = opiegenerator(challenge, "", response))
-		    if (ok != 2)
-		  	PROTOCOL_ERROR
-	    }
-	    else if (opiegenerator(challenge, ctl->password, response))
-		 PROTOCOL_ERROR
-
-	    ok = gen_transact(sock, "PASS %s", response);
-	}
-	else
-#endif /* defined(HAVE_LIBOPIE) && defined(OPIE_ENABLE) */
-	    /* ordinary validation, no one-time password */ 
-	    ok = gen_transact(sock, "PASS %s", ctl->password);
-
-	if (ok != 0)
-	{
-	    if (ok == PS_LOCKBUSY)
-		LOCKBUSY_ERROR
-	    PROTOCOL_ERROR
-	}
-
-	/*
-	 * Empirical experience shows some server/OS combinations
-	 * may need a brief pause even after any lockfiles on the
-	 * server are released, to give the server time to finish
-	 * copying back very large mailfolders from the temp-file...
-	 * this is only ever an issue with extremely large mailboxes.
-	 */
-	sleep(3); /* to be _really_ safe, probably need sleep(5)! */
-	break;
-
-    case P_APOP:
-	if ((gen_transact(sock, "APOP %s %s",
-			  ctl->remotename, ctl->digest)) != 0)
-	    PROTOCOL_ERROR
+	ok = gen_transact(sock, "APOP %s %s", ctl->remotename, ctl->digest);
 	break;
 
     case P_RPOP:
-	if ((gen_transact(sock,"USER %s", ctl->remotename)) != 0)
-	    PROTOCOL_ERROR
-
-	if ((gen_transact(sock, "RPOP %s", ctl->password)) != 0)
-	    PROTOCOL_ERROR
+	if ((ok = gen_transact(sock,"USER %s", ctl->remotename)) == 0)
+	    ok = gen_transact(sock, "RPOP %s", ctl->password);
 	break;
 
     default:
 	error(0, 0, "Undefined protocol request in POP3_auth");
+	ok = PS_ERROR;
     }
 
+    /* maybe we detected a lock-busy condition? */
+    if (ok != 0)
+    {
+        if (ok == PS_LOCKBUSY)
+	{
+	    error(0, 0, "lock busy!  Is another session active?"); 
+	    return(PS_LOCKBUSY);
+	}
+    }
+
+    /*
+     * Empirical experience shows some server/OS combinations
+     * may need a brief pause even after any lockfiles on the
+     * server are released, to give the server time to finish
+     * copying back very large mailfolders from the temp-file...
+     * this is only ever an issue with extremely large mailboxes.
+     */
+    sleep(3); /* to be _really_ safe, probably need sleep(5)! */
+
     /* we're approved */
-    return(0);
+    return(PS_SUCCESS);
 }
 
 static int
@@ -214,6 +243,8 @@ pop3_gettopid( int sock, int num , char *id)
 	    break;
 	if( ! got_it && ! strncasecmp("Message-Id:", buf, 11 )) {
 	    got_it = 1;
+	    /* prevent stack overflows */
+	    buf[IDLEN+12] = 0;
 	    sscanf( buf+12, "%s", id);
 	}
     }
@@ -234,7 +265,7 @@ pop3_slowuidl( int sock,  struct query *ctl, int *countp, int *newp)
      *  + Otherwise run a binary search to determine the last known message
      */
     int ok, nolinear = 0;
-    int first_nr, list_len, try_id, try_nr, hop_id, add_id;
+    int first_nr, list_len, try_id, try_nr, add_id;
     int num;
     char id [IDLEN+1];
     
@@ -255,7 +286,7 @@ pop3_slowuidl( int sock,  struct query *ctl, int *countp, int *newp)
 	    if( (ok = pop3_gettopid( sock, try_id, id )) != 0 )
 		return ok;
     
-	    try_nr = str_nr_in_list(&ctl->oldsaved, id);
+	    try_nr = str_nr_last_in_list(&ctl->oldsaved, id);
 	} else {
 	    try_id = *countp+1;
 	    try_nr = -1;
@@ -318,6 +349,13 @@ static int pop3_getrange(int sock,
     /* Ensure that the new list is properly empty */
     ctl->newsaved = (struct idlist *)NULL;
 
+#ifdef MBOX
+    /* Alain Knaff suggests this, but it's not RFC standard */
+    if (folder)
+	if ((ok = gen_transact(sock, "MBOX %s", folder)))
+	    return ok;
+#endif /* MBOX */
+
     /* get the total message count */
     gen_send(sock, "STAT");
     ok = pop3_ok(sock, buf);
@@ -345,7 +383,10 @@ static int pop3_getrange(int sock,
 	if (ok == 0)
 	{
 	    if (sscanf(buf, "%d", &last) == 0)
-		PROTOCOL_ERROR
+	    {
+		error(0, 0, "protocol error");
+		return(PS_ERROR);
+	    }
 	    *newp = (*countp - last);
 	}
  	else
@@ -355,7 +396,10 @@ static int pop3_getrange(int sock,
 	    {
 		/* don't worry, yet! do it the slow way */
 		if((ok = pop3_slowuidl( sock, ctl, countp, newp))!=0)
-		    PROTOCOL_ERROR
+		{
+		    error(0, 0, "protocol error while fetching UIDLs");
+		    return(PS_ERROR);
+		}
 	    }
 	    else
 	    {
@@ -401,8 +445,6 @@ static int pop3_getsizes(int sock, int count, int *sizes)
 		break;
 	    else if (sscanf(buf, "%d %d", &num, &size) == 2)
 		sizes[num - 1] = size;
-	    else
-		sizes[num - 1] = -1;
 	}
 
 	return(ok);
@@ -424,25 +466,13 @@ static int pop3_fetch(int sock, struct query *ctl, int number, int *lenp)
 /* request nth message */
 {
     int ok;
-    char buf [POPBUFSIZE+1], *cp;
+    char buf [POPBUFSIZE+1];
 
     gen_send(sock, "RETR %d", number);
     if ((ok = pop3_ok(sock, buf)) != 0)
 	return(ok);
 
-    /* 
-     * Look for "nnn octets" -- there may or may not be preceding cruft.
-     * It's OK to punt and pass back -1 as a failure indication here, as 
-     * long as the force_getsizes flag has forced sizes to be preloaded.
-     */
-    if ((cp = strstr(buf, " octets")) == (char *)NULL)
-	*lenp = -1;
-    else
-    {
-	while (--cp >= buf && isdigit(*cp))
-	    continue;
-	*lenp = atoi(++cp);
-    }
+    *lenp = -1;		/* we got sizes from the LIST response */
 
     return(0);
 }
@@ -450,20 +480,19 @@ static int pop3_fetch(int sock, struct query *ctl, int number, int *lenp)
 static int pop3_delete(int sock, struct query *ctl, int number)
 /* delete a given message */
 {
+    /* actually, mark for deletion -- doesn't happen until QUIT time */
     return(gen_transact(sock, "DELE %d", number));
 }
 
-static flag pop3_retain_check(int num, char *buf)
-/* is this a special message that should be retained? */
+static int pop3_logout(int sock, struct query *ctl)
+/* send logout command */
 {
-    /*
-     * The University of Washington IMAP server (the reference
-     * implementation of IMAP4 written by Mark Crispin) relies
-     * on being able to keep base-UID information in a special
-     * message at the head of the mailbox.  This message should
-     * neither be deleted nor forwarded.
-     */
-    return (num == 1 && !strncasecmp(buf, "X-IMAP:", 7));
+    int ok = gen_transact(sock, "QUIT");
+
+    if (!ok)
+	expunge_uids(ctl);
+
+    return(ok);
 }
 
 const static struct method pop3 =
@@ -472,7 +501,6 @@ const static struct method pop3 =
     110,		/* standard POP3 port */
     FALSE,		/* this is not a tagged protocol */
     TRUE,		/* this uses a message delimiter */
-    TRUE,		/* RFC 1725 doesn't require a size field in fetch */
     pop3_ok,		/* parse command response */
     pop3_getauth,	/* get authorization */
     pop3_getrange,	/* query range of messages */
@@ -482,19 +510,22 @@ const static struct method pop3 =
     NULL,		/* no way to fetch body alone */
     NULL,		/* no message trailer */
     pop3_delete,	/* how to delete a message */
-    pop3_retain_check,	/* try not to nuke an IMAP4rev header message */
-    "QUIT",		/* the POP3 exit command */
+    pop3_logout,	/* log out, we're done */
+    FALSE,		/* no, we can't re-poll */
 };
 
 int doPOP3 (struct query *ctl)
 /* retrieve messages using POP3 */
 {
+#ifndef MBOX
     if (ctl->mailboxes->id) {
 	fprintf(stderr,"Option --remote is not supported with POP3\n");
 	return(PS_SYNTAX);
     }
+#endif /* MBOX */
     peek_capable = FALSE;
     return(do_protocol(ctl, &pop3));
 }
+#endif /* POP3_ENABLE */
 
 /* pop3.c ends here */
