@@ -20,13 +20,14 @@
 static int verbose;
 #endif /* TESTMAIN */
 
-void reply_hack(buf, host)
+char *reply_hack(buf, host)
 /* hack message headers so replies will work properly */
 char *buf;		/* header to be hacked */
 const char *host;	/* server hostname */
 {
-    char *from, *cp;
+    char *from, *cp, last_nws = '\0';
     int parendepth, state, has_bare_name_part, has_host_part;
+    int addresscount = 1;
 
     if (strncasecmp("From: ", buf, 6)
 	&& strncasecmp("To: ", buf, 4)
@@ -34,9 +35,17 @@ const char *host;	/* server hostname */
 	&& strncasecmp("Return-Path: ", buf, 13)
 	&& strncasecmp("Cc: ", buf, 4)
 	&& strncasecmp("Bcc: ", buf, 5)) {
-	return;
+	return(buf);
     }
 
+#ifndef TESTMAIN
+    /* make room to hack the address; buf must be malloced */
+    for (cp = buf; *cp; cp++)
+	if (*cp == ',' || isspace(*cp))
+	    addresscount++;
+    buf = (char *)xrealloc(buf, strlen(buf) + addresscount * strlen(host) + 1);
+#endif /* TESTMAIN */
+
     parendepth = state = 0;
     has_host_part = has_bare_name_part = FALSE;
     for (from = buf; *from; from++)
@@ -63,6 +72,8 @@ const char *host;	/* server hostname */
 		break;
 
 	    case 1:	/* we've seen the colon, we're looking for addresses */
+		if (!isspace(*from))
+		    last_nws = *from;
 		if (*from == '<')
 		    state = 3;
 		else if (*from == '@')
@@ -70,15 +81,18 @@ const char *host;	/* server hostname */
 		else if (*from == '"')
 		    state = 2;
 		/*
-		 * Not expanding on from[-1] == ';' deals with groupnames,
+		 * Not expanding on last non-WS == ';' deals with groupnames,
 		 * an obscure misfeature described in sections
 		 * 6.1, 6.2.6, and A.1.5 of the RFC822 standard.
 		 */
-		else if ((*from == ',' || HEADER_END(from)) && has_bare_name_part && !has_host_part && from[-1] != ';')
+		else if ((*from == ',' || HEADER_END(from) || from[1] == '(')
+			 && has_bare_name_part
+			 && !has_host_part
+			 && last_nws != ';')
 		{
 		    int hostlen;
 
-		    while (isspace(*from))
+		    while (isspace(*from) || (*from == ','))
 			--from;
 		    from++;
 		    hostlen = strlen(host);
@@ -86,7 +100,7 @@ const char *host;	/* server hostname */
 			cp[hostlen+1] = *cp;
 		    *from++ = '@';
 		    memcpy(from, host, hostlen);
-		    from += strlen(from);
+		    from += hostlen;
 		    has_host_part = TRUE;
 		} 
 		else if (!isspace(*from))
@@ -101,21 +115,34 @@ const char *host;	/* server hostname */
 	    case 3:	/* we're in a <>-enclosed address */
 		if (*from == '@')
 		    has_host_part = TRUE;
-		else if (*from == '>' && !has_host_part)
+		else if (*from == '>')
 		{
-		    int hostlen;
+		    state = 1;
+		    if (!has_host_part)
+		    {
+			int hostlen;
 
-		    hostlen = strlen(host);
-		    for (cp = from + strlen(from); cp >= from; --cp)
-			cp[hostlen+1] = *cp;
-		    *from++ = '@';
-		    memcpy(from, host, hostlen);
-		    from += strlen(from);
-		    has_host_part = TRUE;
+			hostlen = strlen(host);
+			for (cp = from + strlen(from); cp >= from; --cp)
+			    cp[hostlen+1] = *cp;
+			*from++ = '@';
+			memcpy(from, host, hostlen);
+			from += hostlen;
+			has_host_part = TRUE;
+		    }
 		}
 		break;
 	    }
+
+	/*
+	 * If we passed a comma, reset everything.
+	 */
+	if (from[-1] == ',' && !parendepth) {
+	  has_host_part = has_bare_name_part = FALSE;
+	}
     }
+
+    return(buf);
 }
 
 char *nxtaddr(hdr)
@@ -163,9 +190,12 @@ const char *hdr;	/* header to be parsed, NUL to continue previous hdr */
 	else if (HEADER_END(hp))
 	{
 	    state = ENDIT_ALL;
-	    while (isspace(*--tp))
-		continue;
-	    *++tp = '\0';
+	    if (tp > address)
+	    {
+		while (isspace(*--tp))
+		    continue;
+		*++tp = '\0';
+	    }
 	    return(tp > address ? (tp = address) : (char *)NULL);
 	}
 	else if (*hp == '\\')		/* handle RFC822 escaping */
