@@ -7,6 +7,9 @@
 #include "config.h"
 #include <stdio.h>
 #include <string.h>
+#ifdef HAVE_MEMORY_H
+#include <memory.h>
+#endif /* HAVE_MEMORY_H */
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
@@ -25,6 +28,7 @@
 #endif
 #include "socket.h"
 
+#ifndef INET_ATON
 #ifndef  INADDR_NONE
 #ifdef   INADDR_BROADCAST
 #define  INADDR_NONE	INADDR_BROADCAST
@@ -32,29 +36,39 @@
 #define	 INADDR_NONE	-1
 #endif
 #endif
+#endif /* INET_ATON */
 
-#ifdef SUNOS
-#include <memory.h>
-#endif
-
-int SockOpen(char *host, int clientPort)
+int SockOpen(const char *host, int clientPort)
 {
     int sock;
+#ifndef INET_ATON
     unsigned long inaddr;
+#endif /* INET_ATON */
     struct sockaddr_in ad;
     struct hostent *hp;
 
     memset(&ad, 0, sizeof(ad));
     ad.sin_family = AF_INET;
 
+    /* we'll accept a quad address */
+#ifndef INET_ATON
     inaddr = inet_addr(host);
     if (inaddr != INADDR_NONE)
         memcpy(&ad.sin_addr, &inaddr, sizeof(inaddr));
     else
+#else
+    if (!inet_aton(host, &ad.sin_addr))
+#endif /* INET_ATON */
     {
         hp = gethostbyname(host);
-        if (hp == NULL)
+
+	/*
+	 * Add a check to make sure the address has a valid IPv4 or IPv6
+	 * length.  This prevents buffer spamming by a broken DNS.
+	 */
+        if (hp == NULL || (hp->h_length != 4 && hp->h_length != 8))
             return -1;
+
         memcpy(&ad.sin_addr, hp->h_addr, hp->h_length);
     }
     ad.sin_port = htons(clientPort);
